
// pixel related presets
const PIXEL_X_COUNT   = 64;
const PIXEL_Y_COUNT   = 64;
const PIXELS_PER_LINE = 8; // ashould be 8

const LINES_Y_COUNT   = PIXEL_Y_COUNT / PIXELS_PER_LINE;
const PIXEL_X_MASK    = PIXEL_X_COUNT-1;
const PIXEL_Y_MASK    = PIXEL_Y_COUNT-1;


enum BUSENUM
{
	// BUSBITS.data  [port 888]
	BUS_DB0 =   1,
	BUS_DB1 =   2,
	BUS_DB2 =   4,
	BUS_DB3 =   8,
	BUS_DB4 =  16,
	BUS_DB5 =  32,
	BUS_DB6 =  64,
	BUS_DB7 = 128,
	// BUSBITS.commands [port 889]
	BUS_DI =    1, // 1:data; 0:instruction
	BUS_RW =    2, // 1:read; 0:write
	BUS_E  =    4, // enable
	BUS_RESET = 8,
	BUS_ADC =  16, // 
	BUS_CS1 =  32, // must be 0
	BUS_CS2 =  64, // must be 0
	BUS_CS3 = 128, // must be 1
	// BUSBITS.signals2
	BUS_FRESET  =  16,
	BUS_FDISPON =  32, // display is 1:ON; 0:OFF
	BUS_BUSY   = 128

//	BUS_DATAMASK   = 255,
//	BUS_CONFIGMASK = 768
}
const BUS_DIRWMASK      = BUS_DI | BUS_RW;
const BUS_READDATA      = BUS_DI | BUS_RW;
const BUS_WRITEDATA     = BUS_DI;
const BUS_READSTATUS    = BUS_RW;
const BUS_INSTRUCTION   = 0;
// instructions
const BUS_ISET_Z_LINE      = BUS_DB7 | BUS_DB6;
const BUS_ISET_Z_LINE_MASK = BUS_DB7 | BUS_DB6;
const BUS_ISETPAGE         = BUS_DB7 | BUS_DB5 | BUS_DB4 | BUS_DB3;
const BUS_ISETPAGE_MASK    = BUS_ISETPAGE | BUS_DB6;
const BUS_ISETLINE         = BUS_DB6;
const BUS_ISETLINE_MASK    = BUS_DB7 | BUS_DB6;
const BUS_ILCDCONOF        = BUS_DB5|BUS_DB4|BUS_DB3|BUS_DB2|BUS_DB1;
const BUS_ILCDCONOF_MASK   = BUS_ILCDCONOF | BUS_DB7 | BUS_DB6;

const BUS_CSMASK = BUS_CS1 | BUS_CS2 | BUS_CS3;

// chip select
const CHIPSELECT0 = 0<<5; // 000
const CHIPSELECT1 = 1<<5; // 001
const CHIPSELECT2 = 2<<5; // 010
const CHIPSELECT3 = 3<<5; // 011
const CHIPSELECT4 = 4<<5; // 100
const CHIPSELECT5 = 5<<5; // 101
const CHIPSELECT6 = 6<<5; // 110
const CHIPSELECT7 = 7<<5; // 111


struct BUSBITS,1
{
	BUSENUM all[0];
	UCHAR datain;
	UCHAR dataout;
	UCHAR commands; // must be 100xxxxx -> (commands&BUS_CSMASK) = BUS_CSMASK
	UCHAR signals2;
}

struct COUNTER
{
	int x, y, z;
}

struct DRAWDATA
{
	HDC      dc;
	POINT    ptPos;
	COLORREF bkColor;
	COLORREF crColorOn;
	COLORREF crColorOff;
	int      pixelSize;
	int      pixelSpace;
	RECT     margins; // size of left, top, right and bottom margin
}

class HD61202U
{
	declare HD61202U();
	declare _HD61202U();

	declare HandleException(EXCEPTION_RECORD *ExceptionRecord, CONTEXT *ContextRecord,BOOL *isValid),EXCEPTION_DISPOSITION;
	declare ThreadProc();

	declare Reset();
	declare SetData(int data); // m_bus; BUS_DB0, BUS_DB1, BUS_DB2, BUS_DB3, BUS_DB4, BUS_DB5, BUS_DB6, BUS_DB7
	declare SetBus(int data);  // m_bus; BUS_DI, BUS_RW

	declare cmdReadRam();
	declare cmdWriteRam();
	declare cmdReadStatus();
	declare Instruction();
	declare IncrementAddressCounter();
	// instructions
	declare cmdDisplayOnOff(BOOL on);
	declare cmdDisplayStartLine(int line); // todo
	declare cmdSetPage(int page);
	declare cmdSetLine(int line);


	HD61202U *next;
	BUSBITS  m_bus;
	int      m_lptDataPort;
	int      m_lptSignalPort;
	int      m_ChipSelect;   // 0 - 7
	COUNTER  m_counter; // y:column; x:row -> m_ram[y,x]
	HANDLE   m_hThread;
	HANDLE   m_hDisplayReadyEvent;
	DRAWDATA m_drawdata;
	SIZE     m_lcdSize;

	UCHAR   m_ram[PIXEL_X_COUNT,LINES_Y_COUNT];
	// pixels {0,0} - {0,7}  are at ram[0,0]
	// pixels {1,0} - {1,7}  are at ram[1,0]
	// pixels {0,8} - {0,15} are at ram[0,8]
}

